==> osmos-android-sdk-2.0.2-sources.jar: com/ai/osmos/ads/views/BannerAdView.kt <==
package com.ai.osmos.ads.views

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import com.ai.osmos.ads.renderer.loaders.ImageLoader
import com.ai.osmos.ads.renderer.loaders.VideoLoader
import com.ai.osmos.ads.views.interfaces.BannerAdViewInterface
import com.ai.osmos.core.Config
import com.ai.osmos.models.ads.AdType
import com.ai.osmos.models.ads.BaseAd
import com.ai.osmos.models.ads.ImageAd
import com.ai.osmos.models.ads.VideoAd
import com.ai.osmos.tracking.tracker.PersistentAdTracker
import com.ai.osmos.utils.error.ErrorCallback
import com.ai.osmos.utils.ui.ViewUtils
import com.ai.osmos.utils.ui.toMap
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * Project Name: OSMOS-Android-SDK
 * File Name: AdView
 */

/**
 * BannerAdView for displaying banner advertisements.
 *
 * @param context Application context
 * @param adData Map containing banner ad data structure
 * @param width Optional width of banner ad view in pixels
 * @param height Optional height of banner ad view in pixels
 * @param attrs Optional attribute set for ad view styling
 * @param errorCallback Optional callback for handling errors
 * @param config SDK configuration object
 *
 * Example:
 * ```
 * val adView = BannerAdView(
 *     context = this@MainActivity,
 *     adData = adData,
 *     width = 300,
 *     height = 200,
 *     errorCallback = object : ErrorCallback {
 *         override fun onError(errorCode: String, errorMessage: String, throwable: Throwable?) {
 *             Log.e("BannerAd", "Error: $errorCode - $errorMessage")
 *         }
 *     },
 *     config = config
 * )
 * ```
 */
@SuppressLint("ViewConstructor")
internal class BannerAdView @JvmOverloads constructor(
    context: Context,
    adData: Map<String, Any>,
    private var width: Int? = null,
    private var height: Int? = null,
    attrs: AttributeSet? = null,
    errorCallback: ErrorCallback? = null,
    private val config: Config
) : BaseAdView(context, attrs, config), BannerAdViewInterface {

    private val imageLoader = ImageLoader(coroutineScope, adTracker)
    private val videoLoader = VideoLoader(coroutineScope, adTracker)

    // Initialize the view by loading ad or showing an empty placeholder
    init {
        errorCallback?.let {
            setErrorCallback(it)
            (adTracker as? PersistentAdTracker)?.setErrorCallback(it)
            videoLoader.setErrorCallback(it)
        }
        if (adData.isNotEmpty()) loadAd(adData, context) else showEmptyView(context)
    }

    /**
     * Loads and displays an ad as a banner based on the given ad data.
     *
     * @param adData A map containing the ad object and its associated metadata.
     *               Expected to include a key `"ad"` with value of type `BaseAd`.
     */
    override fun loadAd(adData: Map<String, Any>, context: Context) {
        coroutineScope.launch(Dispatchers.IO) {
            try {
                val ad = adData["ad"] as? BaseAd

                if (ad != null) {
                    val cliUbid = getCliUbid(ad)

                    if (isSupportedCrtImage(ad.crt) || isSupportedCrtVideo(ad.crt)) {
                        withContext(Dispatchers.Main) {
                            removeAllViews()

                            when (ad.elementsType) {
                                //
                                AdType.IMAGE -> {
                                    val imageAd = ad as? ImageAd
                                    val imageUrl = imageAd?.elements?.value
                                    if (!imageUrl.isNullOrEmpty()) {
                                        if (isSupportedImageFormat(ad)) {

                                            val (screenWidth, screenHeight) = getScreenHeightWidth()

                                            val h =
                                                height ?: (imageAd.elements.height ?: screenHeight)
                                            val w = width ?: (imageAd.elements.width ?: screenWidth)

                                            val result = setMaxHeightWidth(h, w, null)

                                            val (width, height) = getAdDimensions(
                                                result.finalWidth,
                                                result.finalHeight,
                                                imageAd.elements.width ?: screenWidth,
                                                imageAd.elements.height ?: screenHeight
                                            )
                                            val imageView = imageLoader.createImageView(
                                                context,
                                                width,
                                                height,
                                                result.isClamped
                                            )

                                            imageLoader.setViewLoadListener { loadedAd, cli_ubid ->
                                                onViewLoadListener?.invoke(
                                                    loadedAd,
                                                    cli_ubid
                                                ) // Notify the listener
                                            }

                                            imageLoader.loadImage(
                                                imageAd,
                                                imageView,
                                                cliUbid,
                                                false,
                                            ) {
                                                adClickedListener?.invoke(imageAd.toMap()) // Pass adMetadata when clicked
                                            }
                                            addView(imageView)
                                        } else {
                                            showEmptyView(context)
                                        }
                                    } else showEmptyView(context)
                                }

                                AdType.VIDEO -> {
                                    val videoAd = ad as? VideoAd
                                    if (videoAd != null && videoAd.elements.value.isNotEmpty()) {
                                        if (isSupportedVideoFormat(ad)) {

                                            val (screenWidth, screenHeight) = getScreenHeightWidth()

                                            val h =
                                                height ?: (videoAd.elements.height ?: screenHeight)
                                            val w = width ?: (videoAd.elements.width ?: screenWidth)

                                            val result = setMaxHeightWidth(h, w, null)

                                            val (width, height) = getAdDimensions(
                                                result.finalWidth,
                                                result.finalHeight,
                                                videoAd.elements.width ?: screenWidth,
                                                videoAd.elements.height ?: screenHeight
                                            )
                                            val videoView = videoAd.let {

                                                videoLoader.createVideoView(
                                                    context, it,
                                                    result.isClamped,
                                                    width,
                                                    height,
                                                    cliUbid
                                                ) {
                                                    adClickedListener?.invoke(videoAd.toMap()) // Pass adMetadata when clicked
                                                }
                                            }

                                            //Set the video view in center of framelayout
                                            videoView.let {
                                                val finalWidth =
                                                    if (result.isClamped) width else ViewUtils.dpToPx(
                                                        context,
                                                        width
                                                    )
                                                val finalHeight =
                                                    if (result.isClamped) height else ViewUtils.dpToPx(
                                                        context,
                                                        height
                                                    )

                                                val layoutParams = LayoutParams(
                                                    finalWidth,
                                                    finalHeight
                                                ).apply {
                                                    gravity = Gravity.CENTER
                                                }
                                                it.layoutParams = layoutParams
                                                addView(it)

                                            } ?: showEmptyView(context)
                                        } else showEmptyView(context)
                                    } else showEmptyView(context)
                                }

                                AdType.CAROUSEL -> {}
                                AdType.MULTI_AD -> {}
                                AdType.UNKNOWN -> {}
                            }
                        }
                    } else {
                        withContext(Dispatchers.Main) {
                            showEmptyView(context)
                        }
                    }
                } else {
                    withContext(Dispatchers.Main) {
                        showEmptyView(context)
                    }
                }
            } catch (e: Exception) {
                withContext(Dispatchers.Main) {
                    showEmptyView(context)
                }
            }
        }
    }


    override fun onDetachedFromWindow() {
        videoLoader.safeReleaseAllPlayers()
        imageLoader.setViewLoadListener { _, _ -> }
        imageLoader.clearViewLoadListener()
        clearAllErrorCallbacks()
        super.onDetachedFromWindow()
    }

    /**
     * Clears all error callbacks to prevent memory leaks.
     */
    private fun clearAllErrorCallbacks() {
        videoLoader.clearErrorCallback()
        (adTracker as? PersistentAdTracker)?.setErrorCallback(null)
        clearErrorCallback()
    }

    override fun loadAd(adData: Map<String, Any>, activity: Activity?) {
    }

    /**
     * Returns the underlying View for adding to view hierarchy
     */
    override fun getView(): View = this
}

==> osmos-android-sdk-2.2.0-sources.jar: com/ai/osmos/ads/views/BannerAdView.kt <==
package com.ai.osmos.ads.views

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import com.ai.osmos.ads.renderer.loaders.ImageLoader
import com.ai.osmos.ads.renderer.loaders.VideoLoader
import com.ai.osmos.ads.views.interfaces.BannerAdViewInterface
import com.ai.osmos.core.Config
import com.ai.osmos.models.ads.AdType
import com.ai.osmos.models.ads.BaseAd
import com.ai.osmos.models.ads.ImageAd
import com.ai.osmos.models.ads.VideoAd
import com.ai.osmos.tracking.tracker.PersistentAdTracker
import com.ai.osmos.utils.error.ErrorCallback
import com.ai.osmos.utils.ui.ViewUtils
import com.ai.osmos.utils.ui.toMap
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * Project Name: OSMOS-Android-SDK
 * File Name: AdView
 */

/**
 * BannerAdView for displaying banner advertisements.
 *
 * @param context Application context
 * @param adData Map containing banner ad data structure
 * @param width Optional width of banner ad view in pixels
 * @param height Optional height of banner ad view in pixels
 * @param attrs Optional attribute set for ad view styling
 * @param errorCallback Optional callback for handling errors
 * @param config SDK configuration object
 *
 * Example:
 * ```
 * val adView = BannerAdView(
 *     context = this@MainActivity,
 *     adData = adData,
 *     width = 300,
 *     height = 200,
 *     errorCallback = object : ErrorCallback {
 *         override fun onError(errorCode: String, errorMessage: String, throwable: Throwable?) {
 *             Log.e("BannerAd", "Error: $errorCode - $errorMessage")
 *         }
 *     },
 *     config = config
 * )
 * ```
 */
@SuppressLint("ViewConstructor")
internal class BannerAdView @JvmOverloads constructor(
    context: Context,
    adData: Map<String, Any>,
    private var width: Int? = null,
    private var height: Int? = null,
    private val adLabelText: String? = null,
    private val adLabelAlignment: Int? = null,
    errorCallback: ErrorCallback? = null,
    private val config: Config,
    attrs: AttributeSet? = null,
) : BaseAdView(context, attrs, config), BannerAdViewInterface {

    private val imageLoader = ImageLoader(coroutineScope, adTracker)
    private val videoLoader = VideoLoader(coroutineScope, adTracker)

    // Initialize the view by loading ad or showing an empty placeholder
    init {
        errorCallback?.let {
            setErrorCallback(it)
            (adTracker as? PersistentAdTracker)?.setErrorCallback(it)
            videoLoader.setErrorCallback(it)
        }
        if (adData.isNotEmpty()) loadAd(adData, context) else showEmptyView(context)
    }

    /**
     * Loads and displays an ad as a banner based on the given ad data.
     *
     * @param adData A map containing the ad object and its associated metadata.
     *               Expected to include a key `"ad"` with value of type `BaseAd`.
     */
    override fun loadAd(adData: Map<String, Any>, context: Context) {
        coroutineScope.launch(Dispatchers.IO) {
            try {
                val ad = adData["ad"] as? BaseAd

                if (ad != null) {
                    val cliUbid = getCliUbid(ad)

                    if (isSupportedCrtImage(ad.crt) || isSupportedCrtVideo(ad.crt)) {
                        withContext(Dispatchers.Main) {
                            removeAllViews()

                            when (ad.elementsType) {
                                //
                                AdType.IMAGE -> {
                                    val imageAd = ad as? ImageAd
                                    val imageUrl = imageAd?.elements?.value
                                    if (!imageUrl.isNullOrEmpty()) {
                                        if (isSupportedImageFormat(ad)) {

                                            val (screenWidth, screenHeight) = getScreenHeightWidth()

                                            val h =
                                                height ?: (imageAd.elements.height ?: screenHeight)
                                            val w = width ?: (imageAd.elements.width ?: screenWidth)

                                            val result = setMaxHeightWidth(h, w, null)

                                            val (width, height) = getAdDimensions(
                                                result.finalWidth,
                                                result.finalHeight,
                                                imageAd.elements.width ?: screenWidth,
                                                imageAd.elements.height ?: screenHeight
                                            )
                                            val imageView = imageLoader.createImageView(
                                                context,
                                                width,
                                                height,
                                                result.isClamped
                                            )

                                            imageLoader.setViewLoadListener { loadedAd, cli_ubid ->
                                                onViewLoadListener?.invoke(
                                                    loadedAd.toMap(),
                                                    cli_ubid
                                                ) // Notify the listener
                                            }

                                            imageLoader.loadImage(
                                                imageAd,
                                                imageView,
                                                cliUbid,
                                                false,
                                            ) {
                                                adClickedListener?.invoke(imageAd.toMap()) // Pass adMetadata when clicked
                                            }
                                            if (!adLabelText.isNullOrEmpty()) {
                                                val container =
                                                    ViewUtils.createAdIndicatorContainer(
                                                        context,
                                                        adLabelText,
                                                        adLabelAlignment,
                                                        imageView.layoutParams.width,
                                                        imageView.layoutParams.height,
                                                        imageView, null
                                                    )
                                                addView(container)
                                            } else {
                                                addView(imageView)
                                            }
                                        } else {
                                            showEmptyView(context)
                                        }
                                    } else showEmptyView(context)
                                }

                                AdType.VIDEO -> {
                                    val videoAd = ad as? VideoAd
                                    if (videoAd != null && videoAd.elements.value.isNotEmpty()) {
                                        if (isSupportedVideoFormat(ad)) {

                                            val (screenWidth, screenHeight) = getScreenHeightWidth()

                                            val h =
                                                height ?: (videoAd.elements.height ?: screenHeight)
                                            val w = width ?: (videoAd.elements.width ?: screenWidth)

                                            val result = setMaxHeightWidth(h, w, null)

                                            val (width, height) = getAdDimensions(
                                                result.finalWidth,
                                                result.finalHeight,
                                                videoAd.elements.width ?: screenWidth,
                                                videoAd.elements.height ?: screenHeight
                                            )
                                            val videoView = videoAd.let {

                                                videoLoader.createVideoView(
                                                    context, it,
                                                    result.isClamped,
                                                    width,
                                                    height,
                                                    cliUbid,
                                                    adClickedListener, // Pass adMetadata when clicked
                                                    onViewLoadListener = onViewLoadListener
                                                )
                                            }

                                            //Set the video view in center of framelayout
                                            videoView.let {
                                                val finalWidth =
                                                    if (result.isClamped) width else ViewUtils.dpToPx(
                                                        context,
                                                        width
                                                    )
                                                val finalHeight =
                                                    if (result.isClamped) height else ViewUtils.dpToPx(
                                                        context,
                                                        height
                                                    )

                                                val layoutParams =
                                                    LayoutParams(finalWidth, finalHeight).apply {
                                                        gravity = Gravity.CENTER
                                                    }
                                                videoView.layoutParams = layoutParams

                                                if (!adLabelText.isNullOrEmpty()) {
                                                    val container =
                                                        ViewUtils.createAdIndicatorContainer(
                                                            context,
                                                            adLabelText,
                                                            adLabelAlignment,
                                                            videoView.layoutParams.width,
                                                            videoView.layoutParams.height,
                                                            null, videoView
                                                        )
                                                    addView(container)
                                                } else {
                                                    addView(videoView)
                                                }

                                            } ?: showEmptyView(context)
                                        } else showEmptyView(context)
                                    } else showEmptyView(context)
                                }

                                AdType.CAROUSEL -> {}
                                AdType.MULTI_AD -> {}
                                AdType.NATIVE_AD -> {}
                                AdType.PDA_AD -> {}
                                AdType.UNKNOWN -> {}
                            }
                        }
                    } else {
                        withContext(Dispatchers.Main) {
                            showEmptyView(context)
                        }
                    }
                } else {
                    withContext(Dispatchers.Main) {
                        showEmptyView(context)
                    }
                }
            } catch (e: Exception) {
                withContext(Dispatchers.Main) {
                    showEmptyView(context)
                }
            }
        }
    }

    override fun loadAd(adData: Map<String, Any>, activity: Activity?) {
    }

    override fun onDetachedFromWindow() {
        videoLoader.safeReleaseAllPlayers()
        imageLoader.setViewLoadListener { _, _ -> }
        imageLoader.clearViewLoadListener()
        clearAllErrorCallbacks()
        super.onDetachedFromWindow()
    }

    /**
     * Clears all error callbacks to prevent memory leaks.
     */
    private fun clearAllErrorCallbacks() {
        videoLoader.clearErrorCallback()
        (adTracker as? PersistentAdTracker)?.setErrorCallback(null)
        clearErrorCallback()
    }

    /**
     * Returns the underlying View for adding to view hierarchy
     */
    override fun getView(): View = this
}
